import { render, screen } from '@testing-library/react'
import React from 'react'
import { ScopedBlock } from './ScopedBlock'
import { MissingScopeAction } from './models'

test('Scoped Block does not render anything if action is hide and there is no scope', async () => {
  const mockAuthActions = { hasScope: jest.fn().mockReturnValue(false) }
  render(
    <ScopedBlock
      authActions={mockAuthActions}
      missingScopeAction={MissingScopeAction.HIDE}
      scope={'room:list'}
      superScope={['room:admin']}
    >
      <div>
        <button>List rooms</button>
      </div>
    </ScopedBlock>
  )
  expect(screen.queryByRole('button', { name: 'List rooms' })).not.toBeInTheDocument()
})

test('Scoped Block disables buttons in children and grand children elements too if action is disabled and there is no scope', async () => {
  const mockAuthActions = { hasScope: jest.fn().mockReturnValue(false) }
  render(
    <ScopedBlock
      authActions={mockAuthActions}
      missingScopeAction={MissingScopeAction.DISABLE}
      scope={'room:list'}
      superScope={['room:admin']}
    >
      <div>
        <button>List rooms grandchildren</button>
      </div>
      <button>List rooms children</button>
    </ScopedBlock>
  )
  expect(screen.getByRole('button', { name: 'List rooms grandchildren' })).toBeDisabled()
  expect(screen.getByRole('button', { name: 'List rooms children' })).toBeDisabled()
})

test('Scoped Block does not do anything if action is ignore and there is no scope', async () => {
  const mockAuthActions = { hasScope: jest.fn().mockReturnValue(false) }
  render(
    <ScopedBlock
      authActions={mockAuthActions}
      missingScopeAction={MissingScopeAction.IGNORE}
      scope={'room:list'}
      superScope={['room:admin']}
    >
      <div>
        <button>List rooms grandchildren</button>
      </div>
      <button>List rooms children</button>
    </ScopedBlock>
  )
  expect(screen.getByRole('button', { name: 'List rooms grandchildren' })).toBeEnabled()
  expect(screen.getByRole('button', { name: 'List rooms children' })).toBeEnabled()
})

test('Scoped Block does not do anything if action is hide and there is scope', async () => {
  const mockAuthActions = { hasScope: jest.fn().mockReturnValue(true) }
  render(
    <ScopedBlock
      authActions={mockAuthActions}
      missingScopeAction={MissingScopeAction.HIDE}
      scope={'room:list'}
      superScope={['room:admin']}
    >
      <div>
        <button>List rooms grandchildren</button>
      </div>
      <button>List rooms children</button>
    </ScopedBlock>
  )
  expect(screen.getByRole('button', { name: 'List rooms grandchildren' })).toBeEnabled()
  expect(screen.getByRole('button', { name: 'List rooms children' })).toBeEnabled()
})

test('Scoped Block does handle text without tag', async () => {
  const mockAuthActions = { hasScope: jest.fn().mockReturnValue(false) }
  render(
    <ScopedBlock
      authActions={mockAuthActions}
      missingScopeAction={MissingScopeAction.DISABLE}
      scope={'room:list'}
      superScope={['room:admin']}
    >
      Text
    </ScopedBlock>
  )
  expect(screen.getByText('Text')).toBeInTheDocument()
})

test('Scoped Block does not fail if empty', async () => {
  const mockAuthActions = { hasScope: jest.fn().mockReturnValue(false) }
  const { container } = render(
    <ScopedBlock
      authActions={mockAuthActions}
      missingScopeAction={MissingScopeAction.DISABLE}
      scope={'room:list'}
      superScope={['room:admin']}
    ></ScopedBlock>
  )
  expect(container).toBeEmptyDOMElement()
})

test('Scoped Block button will be disabled even if inner Scoped Block should allow it', async () => {
  const mockAuthActionsFalse = { hasScope: jest.fn().mockReturnValue(false) }
  const mockAuthActionsTrue = { hasScope: jest.fn().mockReturnValue(true) }
  render(
    <ScopedBlock
      authActions={mockAuthActionsFalse}
      missingScopeAction={MissingScopeAction.DISABLE}
      scope={'room:list'}
      superScope={['room:admin']}
    >
      <ScopedBlock
        authActions={mockAuthActionsTrue}
        missingScopeAction={MissingScopeAction.DISABLE}
        scope={'room:list'}
        superScope={['room:admin']}
      >
        <button>List rooms grandchildren</button>
      </ScopedBlock>
      <button>List rooms children</button>
    </ScopedBlock>
  )
  expect(screen.getByRole('button', { name: 'List rooms grandchildren' })).toBeDisabled()
  expect(screen.getByRole('button', { name: 'List rooms children' })).toBeDisabled()
})
